/**
 *
 */
package jp.agentec.sinaburocast.action.admin.mail;

import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.text.ParseException;
import java.util.List;

import jp.agentec.sinaburocast.action.AbstractAction;
import jp.agentec.sinaburocast.common.SinaburoConstant;
import jp.agentec.sinaburocast.common.SinaburoConstant.MailSendType;
import jp.agentec.sinaburocast.common.annotation.OperationHistoryAno;
import jp.agentec.sinaburocast.common.util.SinaburoUtil;
import jp.agentec.sinaburocast.dto.MailSendResultDto;
import jp.agentec.sinaburocast.entity.EnqueteMail;
import jp.agentec.sinaburocast.entity.EnqueteMailSendResult;
import jp.agentec.sinaburocast.form.admin.mail.MailResultForm;
import jp.agentec.sinaburocast.service.EnqueteMailSendResultService;
import jp.agentec.sinaburocast.service.EnqueteMailService;
import jp.agentec.sinaburocast.service.EnqueteService;
import jp.agentec.sinaburocast.service.MemberNotifyMailService;
import jp.agentec.sinaburocast.service.NotifyMailSendResultService;

import org.apache.log4j.Logger;
import org.seasar.struts.annotation.ActionForm;
import org.seasar.struts.annotation.Execute;
import org.seasar.struts.util.ResponseUtil;

/**
 *
 * メール送信結果
 * @author kim-hs
 *
 */
public class MailSendResultAction extends AbstractAction {

	private final Logger logger = Logger.getLogger(getClass());

	@ActionForm
	public MailResultForm mailResultForm;

	public EnqueteService enqueteService;

	public EnqueteMailService enqueteMailService;

	public EnqueteMailSendResultService enqueteMailSendResultService;

	public NotifyMailSendResultService notifyMailSendResultService;

	public MemberNotifyMailService memberNotifyMailService;

	/**
	 * 初期表示
	 * メール送信結果検索
	 */
	@Execute(validator = false, reset="clearAll")
	public String index() {
		session.removeAttribute("mailResult");
		session.removeAttribute("mailResultMember");

		List<MailSendResultDto> mailResult = enqueteMailService.findAllOrderByIdResult(null, mailResultForm.pageNo, mailResultForm);
		session.setAttribute("mailResult", mailResult);
		return "/admin/mail/mailSendResult.html";
	}

	@Execute(validator = false, reset = "clearAll")
	public String searchMailSendResult() {
		session.removeAttribute("mailResult");
		session.removeAttribute("mailResultMember");

		List<MailSendResultDto> mailResult = enqueteMailService.findAllOrderByIdResult(null, mailResultForm.pageNo, mailResultForm);
		session.setAttribute("mailResult", mailResult);
		return "/admin/mail/mailSendResult.html";
	}

	/**
	 * メール送信結果検索
	 * ページング
	 * @throws ParseException
	 */
	@Execute(validator = false, urlPattern = "searchResult/{pageNo}", reset="clearPage")
	public String mailSearchResultPage() {
		List<MailSendResultDto> mailResult = enqueteMailService.findAllOrderByIdResult(null, mailResultForm.pageNo, mailResultForm);
		session.setAttribute("mailResult", mailResult);
		return "/admin/mail/mailSendResult.html";

	}

	/**
	 * メール送信結果の詳細
	 */
	@Execute(validator = false, reset="clearAll")
	public String mailSendDetail() {
		List<EnqueteMailSendResult> enqueteMailSendResult = enqueteMailSendResultService.findAllOrderByIdResult(null, mailResultForm.pageNo, mailResultForm);

		if (mailResultForm.type == MailSendType.AUTO) {
			String enqueteName = memberNotifyMailService.findById(Integer.parseInt(mailResultForm.enqueteMailId)).enquete.enqueteName;
			session.setAttribute("enqueteName", enqueteName);
		} else if (mailResultForm.type == MailSendType.FILE_UPLOAD) {
			EnqueteMail mail = enqueteMailService.findById(Integer.parseInt(mailResultForm.enqueteMailId));
			String enqueteName = enqueteService.findById(mail.enqueteId).enqueteName;
			session.setAttribute("enqueteName", enqueteName);
		}

		session.setAttribute("mailResultMember", enqueteMailSendResult);
		return "/admin/mail/mailSendResultDetail.html";
	}

	/**
	 * メール送信結果の詳細
	 * ページング
	 */
	@Execute(validator = false, urlPattern = "mailSendDetail/{pageNo}", reset="clearPage")
	public String mailSendDetailPage() {
		List<EnqueteMailSendResult> enqueteMailSendResult = enqueteMailSendResultService.findAllOrderByIdResult(null, mailResultForm.pageNo, mailResultForm);
		session.setAttribute("mailResultMember", enqueteMailSendResult);
		return "/admin/mail/mailSendResultDetail.html";
	}

	/**
	 * エラーレポートダウンロード
	 * @return
	 * @throws UnsupportedEncodingException
	 */
	@Execute(validator = false)
	@OperationHistoryAno(operationTypeId=SinaburoConstant.OperationType.DOWNLOAD,target="メール送信エラーリスト",detail="ダウンロード",screenId=SinaburoConstant.screenId.CMS0007)
	public String errorReport() throws UnsupportedEncodingException {
		EnqueteMail enqueteMail;
		if(mailResultForm.type == MailSendType.AUTO){
			enqueteMail = notifyMailSendResultService.findByMemberNotifyMailIdExchangeEnqueteMail(Integer.parseInt(mailResultForm.enqueteMailId));
		} else{
			enqueteMail = enqueteMailService.findById(Integer.parseInt(mailResultForm.enqueteMailId));
		}

		List <EnqueteMailSendResult> enqueteMailSendResult = enqueteMailSendResultService.findAllOrderByIdReport(Integer.parseInt(mailResultForm.enqueteMailId), mailResultForm.type);
		StringBuffer str = new StringBuffer();
		str.append("\"メール件名："+enqueteMail.subject+"\"");
		str.append("\r\n");
		str.append("\"配信日時："+SinaburoUtil.convertTimestampToString(enqueteMail.deliveryDate,"yyyy/MM/dd HH:mm:ss")+"\"");
		str.append("\r\n");
		str.append("\"送信件数："+enqueteMail.deliveryTotalNum+"\"");
		str.append("\r\n");
		str.append("\"正常件数："+enqueteMail.deliverySuccessNum+"\"");
		str.append("\r\n");
		str.append("\"エラー件数："+enqueteMail.deliveryFailNum+"\"");
		str.append("\r\n");
		if(mailResultForm.type == MailSendType.GROUP){
			str.append("\"会員ID\"\t");
			str.append("\"氏名\"\t");
		}
		str.append("\"メールアドレス\"\t");
		str.append("\"送信結果\"\t");
		str.append("\"エラー内容\"\r\n");
		for (EnqueteMailSendResult report : enqueteMailSendResult) {
			if(report.member != null){
				str.append("\""+report.memberId+"\"\t");
				str.append("\""+report.member.firstName+" "+report.member.lastName+"\"\t");
			}
			str.append("\""+report.email+"\"\t\"");
			if (report.sendResultCd==0) {
				str.append("正常");
			} else {
				str.append("異常");
			}
			if (report.sendResultCd == 999) {
				if(report.statusCd != null){
					if (report.statusCd.equals("9.9.9")) {
						str.append("\"\t\""+report.sendResult+"\"");
					} else {
						str.append("\"\t\""+report.status+"\"");
					}
				} else {
					str.append("\"\t\""+report.sendResult+"\"");
				}
			} else {
				str.append("\"\t\""+report.sendResult+"\"");
			}

			str.append("\r\n");
		}
		String contents = str.toString().replace("null", "");
		
		//20170508 SJIS文字化け対応
		contents = SinaburoUtil.toSJIS(contents);
		ResponseUtil.download(URLEncoder.encode("member_list.tsv", "UTF-8"), contents.getBytes("Windows-31J"));
		//ResponseUtil.download(URLEncoder.encode("member_list.tsv", "UTF-8"), contents.getBytes("Shift_JIS"));
		//response.setContentType("Content-Type content=\"text/html; charset=\"UTF-8\"");
		return null;
	}
}
