package jp.agentec.sinaburocast.action.user;

import java.io.UnsupportedEncodingException;

import javax.mail.MessagingException;

import jp.agentec.sinaburocast.action.AbstractAction;
import jp.agentec.sinaburocast.common.SinaburoConstant;
import jp.agentec.sinaburocast.common.SinaburoConstant.UserType;
import jp.agentec.sinaburocast.common.util.MailUtil;
import jp.agentec.sinaburocast.common.util.PropertyUtil;
import jp.agentec.sinaburocast.common.util.SinaburoUtil;
import jp.agentec.sinaburocast.common.util.SinaburoViewUtil;
import jp.agentec.sinaburocast.entity.Gift;
import jp.agentec.sinaburocast.entity.GiftExchangeInfo;
import jp.agentec.sinaburocast.entity.Member;
import jp.agentec.sinaburocast.form.user.PointUseForm;
import jp.agentec.sinaburocast.service.GiftExchangeInfoService;
import jp.agentec.sinaburocast.service.GiftService;
import jp.agentec.sinaburocast.service.MemberService;
import jp.agentec.sinaburocast.service.MessageService;
import jp.agentec.sinaburocast.service.PointGetService;

import org.apache.log4j.Logger;
import org.apache.struts.Globals;
import org.apache.struts.action.ActionMessage;
import org.apache.struts.action.ActionMessages;
import org.apache.struts.util.TokenProcessor;
import org.seasar.framework.util.StringUtil;
import org.seasar.struts.annotation.ActionForm;
import org.seasar.struts.annotation.Execute;

/**
 * ポイント利用
 *
 * @author choi-c
 *
 */
public class PointUseAction extends AbstractAction {
	private final Logger logger = Logger.getLogger(getClass());

	@ActionForm
	public PointUseForm pointUseForm;

	public PointGetService pointGetService;

	public MemberService memberService;

	public GiftService giftService;

	public MessageService messageService;

	@Execute(validator = false)
	public String index(){

		Member member = getMember();
		if(member == null){
			return "/user/login/";
		}

		member = memberService.findById(member.memberId);

		//最新のユーザー情報を取得
		super.setAuthenticatedToken(member, UserType.USER);

		//住所、連絡先が未登録ユーザの場合、会員情報変更画面に遷移する。
//		if(member.cityName == null ||
//		   member.areaName == null ||
//		   member.buildingName == null ||
//		   member.telno == null ||
//		   member.zipCode == null
//				){
//			return "/user/userEdit/uEdit.html";
//		}

		if (StringUtil.isBlank(member.zipCode) || member.prefectureId == null || StringUtil.isBlank(member.cityName) || StringUtil.isBlank(member.areaName) || StringUtil.isBlank(member.telno)
				|| StringUtil.isBlank(member.firstName) || StringUtil.isBlank(member.lastName) || StringUtil.isBlank(member.firstNameKana) || StringUtil.isBlank(member.lastNameKana)
				|| member.genderCd == null || StringUtil.isBlank(member.birthday)) {

			ActionMessages messages = (ActionMessages)request.getAttribute(Globals.ERROR_KEY);

			if (messages == null) {
				messages = new ActionMessages();
			}
			messages.add("pointUse", new ActionMessage(SinaburoViewUtil.getMessage("errors.E050")));
			request.setAttribute(Globals.ERROR_KEY, messages);
			session.setAttribute("pointCheckFlg", true);

			return "/user/userEdit/";
		}

		pointUseForm.giftExchangeInfo = giftExchangeInfoService.findGiftExchangeInfo();

		pointUseForm.message = messageService.findById(SinaburoConstant.MessageCode.POINT_USE);


		//ポイント利用画面
        return "/user/pointUse/pointUseIndex.html";
	}

	public GiftExchangeInfoService  giftExchangeInfoService;
	@Execute(validator = false)
	public String pointUse(){

		GiftExchangeInfo giftExchangeInfo = giftExchangeInfoService.findGiftExchangeInfo();

		//if(!(Integer.parseInt(today.substring(6,8)) >= 5 && Integer.parseInt(today.substring(6,8)) <= 15)){
			//addErrorMessage("dayError", "errors.E035");
			//return index();
		if(!(Integer.parseInt(giftExchangeInfo.startDate) <= Integer.parseInt(SinaburoUtil.getToDayString())
		   && 		Integer.parseInt(SinaburoUtil.getToDayString()) <= Integer.parseInt(giftExchangeInfo.endDate) )
				){
			request.setAttribute("errMsg", giftExchangeInfo.expiredMessage);
			return "/error/errorMsg.html";
		}

		//景品取得
		pointUseForm.giftList = giftService.findAllOrderById();

		//ポイント利用画面
        return "/user/pointUse/pointUse.html";
	}




	/**
	 * ポイント利用確認(入力情報確認ボタン押下)
	 * @return
	 */
	@Execute(validator = false)
	public String confirm()  {
		Member member = getMember();
		if(member == null){
			return "/user/login/";
		}

		Gift gift = giftService.findById(Integer.parseInt(pointUseForm.giftId));

		//選択した枚数が保有ポイント数より少ない場合
		if( member.pointNum < gift.requiredPoint){
				addError(SinaburoViewUtil.getMessage("errors.E013"));
			return "/user/pointUse/pointUse.html";
		}
		TokenProcessor.getInstance().saveToken(request);
		//ポイント交換申請確認画面
		return "/user/pointUse/pointUseConfirm.html";
	}


	/**
	 * ポイント交換申請ボタン押下
	 * @return 遷移先パス。
	 * @throws MessagingException
	 * @throws UnsupportedEncodingException
	 */
	@Execute(validator = false, validate = "validate", input="/error/doubleAction.html")
	public String regist() throws UnsupportedEncodingException, MessagingException  {
		Member member = getMember();

		if(member == null){
			return "/user/login/";
		}

		member = memberService.findById(member.memberId);
		Gift gift = giftService.findById(Integer.parseInt(pointUseForm.giftId));

		// 選択した枚数が保有ポイント数より少ない場合
		if (member.pointNum < gift.requiredPoint) {
			getMember().pointNum = member.pointNum;
			addError(SinaburoViewUtil.getMessage("errors.E013"));
			return "/user/pointUse/pointUse";
		}
		//２．ユーザのポイント数より消費ポイント数を減算しＤＢに登録する。
		//３．ポイント交換申請情報をDBに登録する。
		pointGetService.cutDownPointLogic(gift,member);
		getMember().pointNum = member.pointNum;

		//メール送信
		//■メール送信
		try {
			GiftExchangeInfo giftExchangeInfo = giftExchangeInfoService.findGiftExchangeInfo();
			if (!StringUtil.isBlank(member.pcEmail)) {
				MailUtil.send(member.pcEmail, PropertyUtil.getProperty("POINT_USE_MAIL_TITLE"), giftExchangeInfo.mailBody);
			}
			if (!StringUtil.isBlank(member.mbEmail)) {
				MailUtil.send(member.mbEmail, PropertyUtil.getProperty("POINT_USE_MAIL_TITLE"), giftExchangeInfo.mailBody);
			}
		} catch (MessagingException e) {
			logger.warn("Mail send failed." + e.toString());
			addError(SinaburoViewUtil.getMessage("errors.E055"));
			return "/user/pointUse/pointUse.html";
		}
		
		//ポイント利用画面
		return "/user/pointUse/pointUseEnd.html";
	}

	/**
	 * 二度押し検証
	 *
	 * @return
	 */
	public ActionMessages validate() {
		ActionMessages errors = new ActionMessages();

		// トークン確認
   		if (!TokenProcessor.getInstance().isTokenValid(request, true)) {
   			errors.add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage("errors.invalid", "Token"));
    	}

   		return errors;
	}
}