package jp.agentec.sinaburocast.action.user;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.List;

import jp.agentec.sinaburocast.action.AbstractAction;
import jp.agentec.sinaburocast.common.SinaburoConstant;
import jp.agentec.sinaburocast.common.SinaburoConstant.UserType;
import jp.agentec.sinaburocast.common.exception.ExpectedException;
import jp.agentec.sinaburocast.common.exception.SystemException;
import jp.agentec.sinaburocast.common.util.BirthdayUtil;
import jp.agentec.sinaburocast.common.util.SinaburoUtil;
import jp.agentec.sinaburocast.common.util.SinaburoViewUtil;
import jp.agentec.sinaburocast.entity.Member;
import jp.agentec.sinaburocast.entity.Postalcode;
import jp.agentec.sinaburocast.entity.Prefecture;
import jp.agentec.sinaburocast.form.user.UserEditForm;
import jp.agentec.sinaburocast.service.MemberService;
import jp.agentec.sinaburocast.service.PostalcodeService;
import jp.agentec.sinaburocast.service.PrefectureService;

import org.apache.log4j.Logger;
import org.apache.struts.Globals;
import org.apache.struts.action.ActionMessage;
import org.apache.struts.action.ActionMessages;
import org.apache.struts.util.TokenProcessor;
import org.seasar.framework.util.StringUtil;
import org.seasar.struts.annotation.ActionForm;
import org.seasar.struts.annotation.Execute;
import org.seasar.struts.util.RequestUtil;


/**
 * 会員情報編集
 *
 * @author kim-hs
 *
 */
public class UserEditAction extends AbstractAction {

	private final Logger logger = Logger.getLogger(getClass());

	public MemberService memberService;

	public PrefectureService prefectureService;

	public PostalcodeService postalcodeService;

	@ActionForm
	public UserEditForm userEditForm;

	/**
	 * 初期表示
	 *
	 * @return
	 * @throws ParseException
	 */
	@Execute(validator = false)
	public String index() throws ParseException {

		Member memberInfoSession = getMember();
		Integer memberId = memberInfoSession.memberId;

		Member memberInfo = memberService.findById(memberId);

		userEditForm.loginId = memberInfo.loginId;
		userEditForm.firstName = memberInfo.firstName;
		userEditForm.lastName = memberInfo.lastName;
		userEditForm.firstNameKana = memberInfo.firstNameKana;
		userEditForm.lastNameKana = memberInfo.lastNameKana;
		userEditForm.pcEmail = memberInfo.pcEmail;
		userEditForm.mbEmail = memberInfo.mbEmail;
		userEditForm.password = memberInfo.password;
		userEditForm.password2 = memberInfo.password;

		userEditForm.zipCode  = memberInfo.zipCode;
		userEditForm.prefectureId = String.valueOf(memberInfo.prefectureId);
		userEditForm.cityName = memberInfo.cityName;
		userEditForm.areaName = memberInfo.areaName;
		userEditForm.buildingName = memberInfo.buildingName;
		userEditForm.genderCd = String.valueOf(memberInfo.genderCd);
		userEditForm.telNo = memberInfo.telno;

		if (!StringUtil.isBlank(memberInfo.birthday)) {
			userEditForm.birthdayY = memberInfo.birthday.substring(0, 4);//SinaburoUtil.convertDateToString( memberInfo.birthday,SinaburoConstant.Formats.TIMESTAMP_FORMAT_YEAR);
			userEditForm.birthdayM = Integer.parseInt(memberInfo.birthday.substring(4, 6))+"";//Integer.parseInt(SinaburoUtil.convertDateToString( memberInfo.birthday,SinaburoConstant.Formats.TIMESTAMP_FORMAT_MONTH))+"";
			userEditForm.birthdayD = Integer.parseInt(memberInfo.birthday.substring(6, 8))+"";//Integer.parseInt(SinaburoUtil.convertDateToString( memberInfo.birthday,SinaburoConstant.Formats.TIMESTAMP_FORMAT_DAY))+"";
		} else {
			userEditForm.birthdayY = "";
			userEditForm.birthdayM = "";
			userEditForm.birthdayD = "";
		}

		//都道府県
		List<Prefecture> prefectureList = new ArrayList<Prefecture>();

		prefectureList = prefectureService.findAllOrderById();

		session.setAttribute("prefectureList", prefectureList);

		String yy = SinaburoUtil.getTimeStringAs(SinaburoConstant.Formats.TIMESTAMP_FORMAT_YEAR);

		session.setAttribute("birthYYYY", Integer.parseInt(yy));

		return "/user/userEdit/uEdit.html";

	}

	/**
	 * 住所検索
	 *
	 * @return
	 */
	@Execute(validator = false)
	public String addrSearch() {

		String zipCode = userEditForm.zipCode;

		List<Postalcode> postalcodeList = new ArrayList<Postalcode>();
		if (zipCode.length() < 3) {
			postalcodeList = null;
		} else {
			postalcodeList = postalcodeService.findTestListByName(zipCode);
		}
		session.setAttribute("postalcodeList", postalcodeList);

		return "/user/userEdit/addrSrch.html";
	}

	/**
	 * 住所検索(モバイル)
	 *
	 * @return
	 */
	@Execute(validator = true, input = "/user/userEdit/uEdit.html")
	public String addrSearchUseMb() {

		//都道府県
		List<Prefecture> prefectureList = new ArrayList<Prefecture>();

		prefectureList = prefectureService.findAllOrderById();

		session.setAttribute("prefectureList", prefectureList);

		Postalcode postalcode = postalcodeService.findAddressByZipCode(userEditForm.zipCode);
		if (postalcode == null){
			userEditForm.prefectureId = null;
			userEditForm.cityName = null;
			userEditForm.areaName = null;
		}else{
			userEditForm.prefectureId = String.valueOf(postalcode.prefecture1.prefectureId);
			userEditForm.cityName = postalcode.city;
			userEditForm.areaName = postalcode.area;
		}
		userEditForm.buildingName = null;

		return "/user/userEdit/uEdit.html";
	}

	/**
	 * 確認
	 *
	 * @return
	 */
	@Execute(validator = true, input = "/user/userEdit/uEdit.html")
	public String confirm() {

		Member memberInfoSession = getMember();
		Member memberInfo = null;

		// メールアドレス登録チェック
		if(StringUtil.isBlank(userEditForm.pcEmail) && StringUtil.isBlank(userEditForm.mbEmail)) {

			ActionMessages messages = (ActionMessages)request.getAttribute(Globals.ERROR_KEY);

			if (messages == null) {
				messages = new ActionMessages();
			}
			messages.add("Email", new ActionMessage(SinaburoViewUtil.getMessage("errors.E010")));
			request.setAttribute(Globals.ERROR_KEY, messages);

			return "/user/userEdit/uEdit.html";
		}

		//携帯メールチェック
		if (!StringUtil.isBlank(userEditForm.mbEmail)) {
			if (!SinaburoUtil.mobileAddressCheck(userEditForm.mbEmail)) {
				ActionMessages messages = (ActionMessages)request.getAttribute(Globals.ERROR_KEY);
				if (messages == null) {
					messages = new ActionMessages();
				}
				messages.add("mbEmail", new ActionMessage(SinaburoViewUtil.getMessage("errors.email",SinaburoViewUtil.getMessage("labels.mbMail"))));
				request.setAttribute(Globals.ERROR_KEY, messages);
				return "/user/userEdit/uEdit.html";
			}
		}

		//メールアドレス重複チェックPCメール
		if (!userEditForm.pcEmail.trim().equals(memberInfoSession.pcEmail)) {
			if (!StringUtil.isBlank(userEditForm.pcEmail)) {

				if (SinaburoUtil.mobileAddressCheck(userEditForm.pcEmail)) {
					ActionMessages messages = (ActionMessages)request.getAttribute(Globals.ERROR_KEY);
					if (messages == null) {
						messages = new ActionMessages();
					}
					messages.add("pcEmail", new ActionMessage(SinaburoViewUtil.getMessage("errors.email",SinaburoViewUtil.getMessage("labels.pcMail"))));
					request.setAttribute(Globals.ERROR_KEY, messages);
					return "/user/userEdit/uEdit.html";
				}

				memberInfo = memberService.findByPcEmail(userEditForm.pcEmail);
				if (memberInfo != null) {
					ActionMessages messages = (ActionMessages)request.getAttribute(Globals.ERROR_KEY);
					if (messages == null) {
						messages = new ActionMessages();
					}
					messages.add("pcEmail", new ActionMessage(SinaburoViewUtil.getMessage("errors.E014")));
					request.setAttribute(Globals.ERROR_KEY, messages);

					return "/user/userEdit/uEdit.html";

				}
			}
		}

		//メールアドレス重複チェック携帯メール
		if (!userEditForm.mbEmail.trim().equals(memberInfoSession.mbEmail)) {
			if (!StringUtil.isBlank(userEditForm.mbEmail)) {

				memberInfo = memberService.findByMoEmail(userEditForm.mbEmail);
				if (memberInfo != null) {
					ActionMessages messages = (ActionMessages)request.getAttribute(Globals.ERROR_KEY);
					if (messages == null) {
						messages = new ActionMessages();
					}

					messages.add("mbEmail", new ActionMessage(SinaburoViewUtil.getMessage("errors.E015")));
					request.setAttribute(Globals.ERROR_KEY, messages);
					return "/user/userEdit/uEdit.html";
				}
			}
		}

		//ログインID重複チェック
		if (!userEditForm.loginId.trim().equals(memberInfoSession.loginId)) {
			memberInfo = memberService.findByUserId(userEditForm.loginId);

			if (memberInfo != null) {
				ActionMessages messages = (ActionMessages)request.getAttribute(Globals.ERROR_KEY);
				if (messages == null) {
					messages = new ActionMessages();
				}
				messages.add("loginId", new ActionMessage(SinaburoViewUtil.getMessage("errors.E016")));
				request.setAttribute(Globals.ERROR_KEY, messages);
				//userEditForm.loginId = memberInfoSession.loginId;
				return "/user/userEdit/uEdit.html";
			}
		}

		//パスワードチェック
		if (!userEditForm.password.trim().equals(userEditForm.password2.trim())) {
			ActionMessages messages = (ActionMessages)request.getAttribute(Globals.ERROR_KEY);
			if (messages == null) {
				messages = new ActionMessages();
			}
			messages.add("password", new ActionMessage(SinaburoViewUtil.getMessage("errors.E011")));
			request.setAttribute(Globals.ERROR_KEY, messages);
			return "/user/userEdit/uEdit.html";
		}

		//
		if (!StringUtil.isBlank(userEditForm.prefectureId)) {
			Prefecture prefectureInfo = prefectureService.findById(Integer.parseInt(userEditForm.prefectureId));
			if (prefectureInfo != null) {
				userEditForm.prefectureName = prefectureInfo.prefecture;
			}
		}

		//誕生日妥当性チェック
		String birthday = userEditForm.birthdayY + userEditForm.birthdayM + userEditForm.birthdayD;

		if (birthday.length() > 0 && birthday.length() < 11) {
			birthday = userEditForm.birthdayY + SinaburoUtil.LeftPadZero(userEditForm.birthdayM, 2) + SinaburoUtil.LeftPadZero(userEditForm.birthdayD, 2);
			SimpleDateFormat format = new SimpleDateFormat("yyyyMMdd");
			format.setLenient(false);
			try {
				format.parse(birthday);
			} catch (ParseException e) {
				ActionMessages messages = (ActionMessages)request.getAttribute(Globals.ERROR_KEY);
				if (messages == null) {
					messages = new ActionMessages();
				}
				messages.add("birthdayY", new ActionMessage(SinaburoViewUtil.getMessage("errors.invalid",SinaburoViewUtil.getMessage("labels.birthDay"))));
				request.setAttribute(Globals.ERROR_KEY, messages);
				return "/user/userEdit/uEdit.html";
			}
		}
		//寄付、ポイント交換の場合、チェック
		Boolean flg = (Boolean) session.getAttribute("pointCheckFlg");
		flg = flg == null ? false : flg;
		if (flg) {
			if (StringUtil.isBlank(userEditForm.zipCode)) {
				ActionMessages messages = (ActionMessages)request.getAttribute(Globals.ERROR_KEY);
				if (messages == null) {
					messages = new ActionMessages();
				}
				messages.add("zipCode", new ActionMessage(SinaburoViewUtil.getMessage("errors.required",SinaburoViewUtil.getMessage("labels.zipCode"))));
				request.setAttribute(Globals.ERROR_KEY, messages);
				return "/user/userEdit/uEdit.html";
			}
			if (StringUtil.isBlank(userEditForm.prefectureId)) {
				ActionMessages messages = (ActionMessages)request.getAttribute(Globals.ERROR_KEY);
				if (messages == null) {
					messages = new ActionMessages();
				}
				messages.add("prefectureId", new ActionMessage(SinaburoViewUtil.getMessage("errors.required",SinaburoViewUtil.getMessage("labels.prefectureId"))));
				request.setAttribute(Globals.ERROR_KEY, messages);
				return "/user/userEdit/uEdit.html";
			}
			if (StringUtil.isBlank(userEditForm.cityName)) {
				ActionMessages messages = (ActionMessages)request.getAttribute(Globals.ERROR_KEY);
				if (messages == null) {
					messages = new ActionMessages();
				}
				messages.add("cityName", new ActionMessage(SinaburoViewUtil.getMessage("errors.required",SinaburoViewUtil.getMessage("labels.cityName"))));
				request.setAttribute(Globals.ERROR_KEY, messages);
				return "/user/userEdit/uEdit.html";
			}
			if (StringUtil.isBlank(userEditForm.areaName)) {
				ActionMessages messages = (ActionMessages)request.getAttribute(Globals.ERROR_KEY);
				if (messages == null) {
					messages = new ActionMessages();
				}
				messages.add("areaName", new ActionMessage(SinaburoViewUtil.getMessage("errors.required",SinaburoViewUtil.getMessage("labels.areaName"))));
				request.setAttribute(Globals.ERROR_KEY, messages);
				return "/user/userEdit/uEdit.html";
			}
			if (StringUtil.isBlank(userEditForm.telNo)) {
				ActionMessages messages = (ActionMessages)request.getAttribute(Globals.ERROR_KEY);
				if (messages == null) {
					messages = new ActionMessages();
				}
				messages.add("telNo", new ActionMessage(SinaburoViewUtil.getMessage("errors.required",SinaburoViewUtil.getMessage("labels.telNo"))));
				request.setAttribute(Globals.ERROR_KEY, messages);
				return "/user/userEdit/uEdit.html";
			}
			if (StringUtil.isBlank(userEditForm.firstName)) {
				ActionMessages messages = (ActionMessages)request.getAttribute(Globals.ERROR_KEY);
				if (messages == null) {
					messages = new ActionMessages();
				}
				messages.add("firstName", new ActionMessage(SinaburoViewUtil.getMessage("errors.required",SinaburoViewUtil.getMessage("labels.firstName"))));
				request.setAttribute(Globals.ERROR_KEY, messages);
				return "/user/userEdit/uEdit.html";
			}
			if (StringUtil.isBlank(userEditForm.lastName)) {
				ActionMessages messages = (ActionMessages)request.getAttribute(Globals.ERROR_KEY);
				if (messages == null) {
					messages = new ActionMessages();
				}
				messages.add("firstName", new ActionMessage(SinaburoViewUtil.getMessage("errors.required",SinaburoViewUtil.getMessage("labels.lastName"))));
				request.setAttribute(Globals.ERROR_KEY, messages);
				return "/user/userEdit/uEdit.html";
			}
			if (StringUtil.isBlank(userEditForm.firstNameKana)) {
				ActionMessages messages = (ActionMessages)request.getAttribute(Globals.ERROR_KEY);
				if (messages == null) {
					messages = new ActionMessages();
				}
				messages.add("firstNameKana", new ActionMessage(SinaburoViewUtil.getMessage("errors.required",SinaburoViewUtil.getMessage("labels.firstKana"))));
				request.setAttribute(Globals.ERROR_KEY, messages);
				return "/user/userEdit/uEdit.html";
			}
			if (StringUtil.isBlank(userEditForm.lastNameKana)) {
				ActionMessages messages = (ActionMessages)request.getAttribute(Globals.ERROR_KEY);
				if (messages == null) {
					messages = new ActionMessages();
				}
				messages.add("firstNameKana", new ActionMessage(SinaburoViewUtil.getMessage("errors.required",SinaburoViewUtil.getMessage("labels.lastKana"))));
				request.setAttribute(Globals.ERROR_KEY, messages);
				return "/user/userEdit/uEdit.html";
			}
			if (StringUtil.isBlank(userEditForm.genderCd)) {
				ActionMessages messages = (ActionMessages)request.getAttribute(Globals.ERROR_KEY);
				if (messages == null) {
					messages = new ActionMessages();
				}
				messages.add("genderCd", new ActionMessage(SinaburoViewUtil.getMessage("errors.required",SinaburoViewUtil.getMessage("labels.genderCd"))));
				request.setAttribute(Globals.ERROR_KEY, messages);
				return "/user/userEdit/uEdit.html";

			}
			if (StringUtil.isBlank(userEditForm.birthdayY) || StringUtil.isBlank(userEditForm.birthdayM) || StringUtil.isBlank(userEditForm.birthdayD)) {
				ActionMessages messages = (ActionMessages)request.getAttribute(Globals.ERROR_KEY);
				if (messages == null) {
					messages = new ActionMessages();
				}
				messages.add("birthdayY", new ActionMessage(SinaburoViewUtil.getMessage("errors.required",SinaburoViewUtil.getMessage("labels.birthDay"))));
				request.setAttribute(Globals.ERROR_KEY, messages);
				return "/user/userEdit/uEdit.html";
			}

		}
    	// トークン発行
		TokenProcessor.getInstance().saveToken(request);

		session.setAttribute("userEditFormInfo", userEditForm);
		session.setAttribute("pointCheckFlg", false);
		return "/user/userEdit/uEditConfirm.html";
	}

	/**
	 * 前の画面に遷移
	 *
	 * @return
	 */
	@Execute(validator = false)
	public String back() {
		userEditForm = (UserEditForm)session.getAttribute("userEditFormInfo");
		return "/user/userEdit/uEdit.html";
	}

	/**
	 * 会員登録情報更新
	 *
	 * @return
	 * @throws SystemException
	 */
	@Execute(validator = false)
	public String editRegist() throws SystemException  {
		userEditForm = (UserEditForm)session.getAttribute("userEditFormInfo");
		// トークン確認
		if (!TokenProcessor.getInstance().isTokenValid(RequestUtil.getRequest(), true)) {
   			return "/error/doubleAction.html";
		}

		Member member  =  getMember();
		Member memberInfo = memberService.findById(member.memberId);

		member.memberId = member.memberId;
		member.loginId = userEditForm.loginId;
		member.firstName = userEditForm.firstName;
		member.lastName = userEditForm.lastName;
		member.firstNameKana = userEditForm.firstNameKana;
		member.lastNameKana = userEditForm.lastNameKana;
		member.pcEmail = userEditForm.pcEmail;
		member.mbEmail = userEditForm.mbEmail;
		member.password = userEditForm.password;
		member.password = userEditForm.password2;
		member.zipCode = userEditForm.zipCode;
		if (!StringUtil.isBlank(userEditForm.prefectureId)) {
			member.prefectureId = Integer.parseInt(userEditForm.prefectureId);
		} else {
			member.prefectureId = null;
		}
		member.cityName = userEditForm.cityName;
		member.areaName = userEditForm.areaName;
		member.buildingName = userEditForm.buildingName;
		if (StringUtil.isNotBlank(userEditForm.genderCd)) {
			member.genderCd = Integer.parseInt(userEditForm.genderCd);
		} else {
			member.genderCd = null;
		}
		member.telno = userEditForm.telNo;
		member.version = memberInfo.version;
		member.birthday = userEditForm.birthdayY +
				SinaburoUtil.LeftPadZero(userEditForm.birthdayM, 2) +
				SinaburoUtil.LeftPadZero(userEditForm.birthdayD, 2);

		if(!StringUtil.isBlank(userEditForm.zipCode)) {
			if (userEditForm.zipCode.substring(0, 3).equals("272")) {
				member.localCityFlg = 1;
			} else {
				member.localCityFlg = 0;
			}
		}
		memberService.update(member, userEditForm.loginId);

		//セッションに新しい情報設定する。
		setAuthenticatedToken(memberService.findById(member.memberId),UserType.USER);

		return "/user/userEdit/uEditEnd.html";
	}

}
